% loesungen.m
% Kapitel 6: Signalverarbeitung - Signal Processing Toolbox
%            Lsungen zu den bungsaufgaben

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6.6.1 Signaltransformation im Frequenzbereich

t = 0:0.001:0.0499;                   % Zeitvektor   1000 Hz
x = square (2*pi*20*t);               % Rechtecksignal 20 Hz

y = fft (x);                          % DFT (ohne Normierung)
n = length (y);                       % Lnge des Datensatzes
f = (0:n-1) / n / diff(t(1:2));       % Frequenzvektor zum Plotten

                                      % Erzeugt Skalierungsvektor
skal = imag (exp (j*pi/2*(0:n-1)));   % 0 1 0 -1   0 1   0 -1   ...
skal = skal ./ [1 (1:n-1)];           % 0 1 0 -1/3 0 1/5 0 -1/7 ...

y_skal = y .* skal;                   % Skalierte Fourierkoeffizienten
x_skal = ifft (y_skal);               % inverse DFT (ohne Normierung)

figure (1)
subplot (121)
    plot (t, x, 'b--')
    hold on
    plot (t, real (x_skal), 'r-')
    axis ([0 max(t) -1.2 1.2])
    legend ('Rechteck', 'Dreieck', 1)
    title ('Zeitsignale')
    xlabel ('Zeit [s]')
    
subplot (122)
    p1 = stem (f, -2*imag (y), 'bx');
    hold on
    p2 = stem (f, -2*imag (y_skal), 'ro');
    axis ([0 max(f)/2 -10 70])
    legend ([p1(1), p2(1)], 'Rechteck', 'Dreieck', 1)
    title ('Fourierkoeffizienten b_k')
    xlabel ('Frequenz [Hz]')   
    
pause    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6.6.2 Signalanalyse und digitale Filterung

% Der Datensatz wurde folgendermaen erzeugt:

t = 0.001:0.001:1;                                   % Zeitvektor Fs = 1000 Hz
xx = 12 * sin (2*pi*4*t) + ...                       % Sollsignal      4 Hz
      4 * sin (2*pi*12*t);                           %                12 Hz
x =  xx + ...
     20 * cos (2*pi*63*t) + ...                      % Strsignal     63 Hz
     18 * cos (2*pi*137*t) .* sin (2*pi*29*t) +  ... % 137-29 Hz, 137+29 Hz
      4 + randn (size(t));                           % Rauschen

datei = fopen ('ueb_signal.dat', 'w');               % Signal speichern (ASCII)
fprintf (datei, '%5.3f  %6.2f\r\n', [t' x']');       % \r\n = CRLF im DOS-Format
fclose (datei);

save ueb_signal t x                                  % Signal speichern (MAT)

clear

% Lsung der Aufgabe

load ueb_signal.dat                          % Signal laden
t = ueb_signal (:,1)';                       % Zeitvektor
x = ueb_signal (:,2)';                       % Messdaten-Vektor

T = diff (t(1:2));                           % Abtastzeit
N = length (x);                              % Lnge des Datenvektors
f = [0:floor(N/2)] / (N*T);                  % Frequenzvektor fr Plot

H = fft (x);                                 % Fouriertransformation
H = [H(1) 2*H(2:floor(N/2)+1)] / N;          % Begrenzung und Normierung

                                             % FIR-Filter auslegen
ord    = 40;                                 % Ordnung des FIR-Filters
B      = fir1 (ord, 0.04);                   % FIR-Filter berechnen f_g = 20 Hz
x_filt = filter (B, 1, x);                   % Daten filtern
x_filt = x_filt - mean (x_filt);             % Gleichanteil ausblenden

figure (2)
subplot (221)
    plot (t, x, 'b-')
    xlabel ('Zeit [s]')
    title ('Ungefiltertes Signal')
subplot (222)
    stem (f, abs (H), 'bo')
    axis ([0 500 0 25])
    xlabel ('Frequenz [Hz]')
    title ('Spektrum')
subplot (223)
    plot (t, x_filt, 'r-')
    xlabel ('Zeit [s]')
    title ('Gefiltertes Signal')

% Die bertragungsfunktion des FIR-Filters kann angezeigt werden mit:

F   = [-ord/2:0.1:ord/2] / (ord*T);
amp = zeros (size (F));
for i = -ord/2:ord/2
   amp = amp + B(i+ord/2+1) * cos (2*pi*F*i*T);
end
subplot (224)
    plot (F, abs (amp))
    axis ([0 500 0 1])
    xlabel ('Frequenz [Hz]')
    title ('Filter-bertragungsfunktion')

pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6.6.3 Analoger Bandpass

% Es werden ein Bandpass (10...100 Hz) und eine Bandsperre (20...70 Hz)
% erzeugt. Die Faltung beider bertragungsfunktionen ergibt das gewnschte Filter.

[B1, A1] = butter (12, 2*pi*[10 100], 's');        % Bandpass 10...100 Hz
[B2, A2] = butter (12, 2*pi*[20 70], 'stop', 's'); % Bandsperre 20...70 Hz

A = conv (A1, A2);                                 % Nenner Falten
B = conv (B1, B2);                                 % Zhler Falten

W = 2*pi * linspace (0, 300, 301);                 % Frequenzvektor [rad/s]
H = freqs (B, A, W);                               % Gesamt-bertragungsfunktion

figure (3)
    plot ([0 10 10 20 20 70 70 100 100 300], ...       % ideale bertragungs-
          [0  0  1  1  0  0  1   1   0   0], 'k--')    % funktion
    hold on
    plot (W/2/pi, abs (H), 'b-')
    axis ([0 150 0 1.2])
    xlabel ('Frequenz [Hz]')
    title ('Gesamt-Filter')
    legend ('Ideal', '12. Ordnung')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
